@@ -23,13 +23,13 @@
 // To avoid collision with other typenames include the ABSOLUTE MINIMUM
 // complexity of includes here. Use forward declarations wherever possible
 // and hide includes of complex types in baseapi.cpp.
-#include "tess_version.h"
 #include "apitypes.h"
 #include "pageiterator.h"
 #include "platform.h"
 #include "publictypes.h"
 #include "resultiterator.h"
 #include "serialis.h"
+#include "tess_version.h"
 #include "tesscallback.h"
 #include "thresholder.h"
 #include "unichar.h"
@@ -56,7 +56,7 @@ class UNICHARSET;
 class WERD_CHOICE_LIST;
 
 struct INT_FEATURE_STRUCT;
-typedef INT_FEATURE_STRUCT *INT_FEATURE;
+using INT_FEATURE = INT_FEATURE_STRUCT *;
 struct TBLOB;
 
 namespace tesseract {
@@ -73,20 +73,10 @@ class Tesseract;
 class Trie;
 class Wordrec;
 
-typedef int (Dict::*DictFunc)(void* void_dawg_args,
-                              const UNICHARSET& unicharset,
-                              UNICHAR_ID unichar_id, bool word_end) const;
-typedef double (Dict::*ProbabilityInContextFunc)(const char* lang,
-                                                 const char* context,
-                                                 int context_bytes,
-                                                 const char* character,
-                                                 int character_bytes);
-typedef float (Dict::*ParamsModelClassifyFunc)(
-    const char *lang, void *path);
-typedef void (Wordrec::*FillLatticeFunc)(const MATRIX &ratings,
-                                         const WERD_CHOICE_LIST &best_choices,
-                                         const UNICHARSET &unicharset,
-                                         BlamerBundle *blamer_bundle);
+using DictFunc = int (Dict::*)(void *, const UNICHARSET &, UNICHAR_ID, bool) const;
+using ProbabilityInContextFunc = double (Dict::*)(const char *, const char *, int, const char *, int);
+using ParamsModelClassifyFunc = float (Dict::*)(const char *, void *);
+using FillLatticeFunc = void (Wordrec::*)(const MATRIX &, const WERD_CHOICE_LIST &, const UNICHARSET &, BlamerBundle *);
 typedef TessCallback4<const UNICHARSET &, int, PageIterator *, Pix *>
     TruthCallback;
 
@@ -196,8 +186,7 @@ class TESS_API TessBaseAPI {
    * NOTE that the only members that may be called before Init are those
    * listed above here in the class definition.
    *
-   * The datapath must be the name of the parent directory of tessdata and
-   * must end in / . Any name after the last / will be stripped.
+   * The datapath must be the name of the tessdata directory.
    * The language is (usually) an ISO 639-3 string or nullptr will default to eng.
    * It is entirely safe (and eventually will be efficient too) to call
    * Init multiple times on the same instance to change language, or just
@@ -404,7 +393,7 @@ class TESS_API TessBaseAPI {
    * If paraids is not nullptr, the paragraph-id of each line within its block is
    * also returned as an array of one element per line. delete [] after use.
    */
-  Boxa* GetTextlines(const bool raw_image, const int raw_padding,
+  Boxa* GetTextlines(bool raw_image, int raw_padding,
                      Pixa** pixa, int** blockids, int** paraids);
   /*
      Helper method to extract from the thresholded image. (most common usage)
@@ -453,9 +442,9 @@ class TESS_API TessBaseAPI {
    * instead of the thresholded image and padded with raw_padding.
    * If text_only is true, then only text components are returned.
    */
-  Boxa* GetComponentImages(const PageIteratorLevel level,
-                           const bool text_only, const bool raw_image,
-                           const int raw_padding,
+  Boxa* GetComponentImages(PageIteratorLevel level,
+                           bool text_only, bool raw_image,
+                           int raw_padding,
                            Pixa** pixa, int** blockids, int** paraids);
   // Helper function to get binary images with no padding (most common usage).
   Boxa* GetComponentImages(const PageIteratorLevel level,
@@ -597,6 +586,19 @@ int GetOCRWords(int **x0,int **y0,int **x1,int **y1,int **ybaseline,char **utf8w
    */
   char* GetHOCRText(int page_number);
 
+  /**
+   * Make an XML-formatted string with Alto markup from the internal
+   * data structures.
+   */
+  char* GetAltoText(ETEXT_DESC* monitor, int page_number);
+
+
+  /**
+   * Make an XML-formatted string with Alto markup from the internal
+   * data structures.
+   */
+  char* GetAltoText(int page_number);
+
   /**
    * Make a TSV-formatted string from the internal data structures.
    * page_number is 0-based but will appear in the output as 1-based.
@@ -604,6 +606,14 @@ int GetOCRWords(int **x0,int **y0,int **x1,int **y1,int **ybaseline,char **utf8w
    */
   char* GetTSVText(int page_number);
 
+  /**
+   * Make a box file for LSTM training from the internal data structures.
+   * Constructs coordinates in the original image - not just the rectangle.
+   * page_number is a 0-based page index that will appear in the box file.
+   * Returned string must be freed with the delete [] operator.
+   */
+  char* GetLSTMBoxText(int page_number);
+
   /**
    * The recognized text is returned as a char* which is coded in the same
    * format as a box file used in training.
@@ -613,6 +623,14 @@ int GetOCRWords(int **x0,int **y0,int **x1,int **y1,int **ybaseline,char **utf8w
    */
   char* GetBoxText(int page_number);
 
+  /**
+   * The recognized text is returned as a char* which is coded in the same
+   * format as a WordStr box file used in training.
+   * page_number is a 0-based page index that will appear in the box file.
+   * Returned string must be freed with the delete [] operator.
+   */
+  char* GetWordStrBoxText(int page_number);
+
   /**
    * The recognized text is returned as a char* which is coded
    * as UNLV format Latin-1 with specific reject and suspect codes.
